part of masamune_local_messaging;

final localeMessagingProvider = ChangeNotifierProvider(
  (_) => LocalMessagingModel(),
);

class LocalMessagingModel extends Model<void> {
  LocalMessagingModel() : super();

  @protected
  final FlutterLocalNotificationsPlugin messaging =
      FlutterLocalNotificationsPlugin();

  /// True if the billing system has been initialized.
  bool get initialized => _initialized;
  bool _initialized = false;

  Future<LocalMessagingModel> initialize({
    String androidDefaultIcon = "@mipmap/ic_launcher",
  }) async {
    if (Config.isWeb) {
      throw Exception("This platform is not supported.");
    }
    if (initialized) {
      return this;
    }
    await messaging.initialize(
      InitializationSettings(
        android: AndroidInitializationSettings(androidDefaultIcon),
        iOS: const IOSInitializationSettings(),
      ),
    );
    _initialized = true;
    return this;
  }

  Future<void> send({
    required String topic,
    required String title,
    required String text,
    String androidChannelId = "defaultChannel",
    String androidChannelName = "Default Channel",
  }) async {
    if (!initialized) {
      debugPrint(
        "It has not been initialized. Please initialize it by executing [initialize()].",
      );
      return;
    }
    await messaging.show(
      topic.hashCode,
      title,
      text,
      NotificationDetails(
        android: AndroidNotificationDetails(
          androidChannelId,
          androidChannelName.localize(),
          importance: Importance.high,
          priority: Priority.high,
        ),
      ),
    );
  }

  Future<void> schedule({
    required String topic,
    required String title,
    required String text,
    required DateTime dateTime,
    MessagingScheduleRepeatType repeatType = MessagingScheduleRepeatType.none,
    String androidChannelId = "defaultChannel",
    String androidChannelName = "Default Channel",
  }) async {
    if (!initialized) {
      debugPrint(
        "It has not been initialized. Please initialize it by executing [initialize()].",
      );
      return;
    }
    await messaging.zonedSchedule(
      topic.hashCode,
      title,
      text,
      tz.TZDateTime.from(dateTime, tz.UTC),
      NotificationDetails(
        android: AndroidNotificationDetails(
          androidChannelId,
          androidChannelName.localize(),
          importance: Importance.high,
          priority: Priority.high,
        ),
      ),
      matchDateTimeComponents: _convertRepeatType(repeatType),
      androidAllowWhileIdle: true,
      uiLocalNotificationDateInterpretation:
          UILocalNotificationDateInterpretation.absoluteTime,
    );
  }

  DateTimeComponents _convertRepeatType(
    MessagingScheduleRepeatType repeatType,
  ) {
    switch (repeatType) {
      case MessagingScheduleRepeatType.monthly:
        return DateTimeComponents.dayOfMonthAndTime;
      case MessagingScheduleRepeatType.weekly:
        return DateTimeComponents.dayOfWeekAndTime;
      default:
        return DateTimeComponents.dateAndTime;
    }
  }

  Future<void> cancelSchedule(
    String topic,
  ) async {
    if (!initialized) {
      debugPrint(
        "It has not been initialized. Please initialize it by executing [initialize()].",
      );
      return;
    }
    await messaging.cancel(
      topic.hashCode,
    );
  }

  Future<void> cancelAllSchedule() async {
    if (!initialized) {
      debugPrint(
        "It has not been initialized. Please initialize it by executing [initialize()].",
      );
      return;
    }
    await messaging.cancelAll();
  }
}
